unit Main;

interface

uses
  Windows, Messages, SysUtils,
  Classes, Graphics, Controls,
  Forms, Dialogs, DirectX, Menus;

type
  TForm1 = class(TForm)
    MainMenu1: TMainMenu;
    File1: TMenuItem;
    ChangleBackgroundColor1: TMenuItem;
    Exit1: TMenuItem;
    Render1: TMenuItem;
    Flat1: TMenuItem;
    Wireframe1: TMenuItem;
    Gouraud1: TMenuItem;
    Textures1: TMenuItem;
    Flat2: TMenuItem;
    Cylinder1: TMenuItem;
    Sphere1: TMenuItem;
    Rotation1: TMenuItem;
    X1: TMenuItem;
    Y1: TMenuItem;
    Z1: TMenuItem;
    Stop1: TMenuItem;
    OpenDialog1: TOpenDialog;
    OpenDialog2: TOpenDialog;
    ColorDialog1: TColorDialog;
    procedure FormCreate(Sender: TObject);
    procedure FormPaint(Sender: TObject);
    procedure FormResize(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure Flat1Click(Sender: TObject);
    procedure Wireframe1Click(Sender: TObject);
    procedure Gouraud1Click(Sender: TObject);
    procedure Exit1Click(Sender: TObject);
    procedure Flat2Click(Sender: TObject);
    procedure Cylinder1Click(Sender: TObject);
    procedure Sphere1Click(Sender: TObject);
    procedure ChangleBackgroundColor1Click(Sender: TObject);
    procedure X1Click(Sender: TObject);
    procedure Y1Click(Sender: TObject);
    procedure Z1Click(Sender: TObject);
    procedure Stop1Click(Sender: TObject);
    procedure FormActivate(Sender: TObject);
  private
    FDirect3D   :IDirect3DRM;
    FD3DClipper : IDirectDrawClipper;
    FD3Device   : IDirect3DRMDevice;
    FD3DScene   : IDirect3DRMFrame;
    FD3DCamera  : IDirect3DRMFrame;
    FD3DViewPort: IDirect3DRMViewport;
    FD3DMeshBuilder     : IDirect3DRMMeshBuilder;
    FD3DMeshBuilderFrame: IDirect3DRMFrame;
    FD3DTextureWrapType : D3DRMWRAPTYPE;
    procedure CreateDevice;
    procedure IdleEvent(Sender: TObject; var Done: Boolean);
    procedure CreateScene;
    function LoadWrapTexture: Boolean;
    function LoadScene: Boolean;
    procedure ScaleScene(Mesh: IDirect3DRMMeshBuilder; Dim: D3DVALUE);
    procedure ApplyCylinder(MeshBuilder: IDirect3DRMMeshBuilder);
    procedure ApplyFlat(MeshBuilder: IDirect3DRMMeshBuilder);
    procedure ApplySphere(MeshBuilder: IDirect3DRMMeshBuilder);
  public
  end;

var
  Form1: TForm1;

implementation

{$R *.DFM}

procedure TForm1.FormCreate(Sender: TObject);
var
 hr: HResult;
begin
 Application.OnIdle:=IdleEvent;
 hr:=Direct3DRMCreate(FDirect3D);
 if hr<>D3DRM_OK then
  ShowMessage('Direct3DRMCreate failed');
 CreateDevice;
end;

procedure TForm1.FormDestroy(Sender: TObject);
begin
 if (FD3DCamera<>nil) then
  FD3DCamera:=nil;

 if (FD3DViewPort<>nil) then
  FD3DViewPort:=nil;

 if (FD3DScene<>nil) then
  FD3DScene:=nil;

 if (FD3Device<>nil) then
  FD3Device:=nil;

 if (FD3DClipper<>nil) then
  FD3DClipper:=nil;

 if (FDirect3D<>nil) then
  FDirect3D:=nil;
end;

procedure TForm1.IdleEvent(Sender: TObject; var Done: Boolean);
begin
 done:=false;
 if (FDirect3D<>nil) and (FD3Device<>nil) then
  FDirect3D.Tick(0.2);
end;

procedure TForm1.FormPaint(Sender: TObject);
var
 D3DWinDevice   : IDirect3DRMWinDevice;
 hr             : HResult;
begin
 if (FD3Device<>nil) then
  begin
   hr:=FD3Device.QueryInterface(IID_IDirect3DRMWinDevice, D3DWinDevice);
   if hr=DD_OK then
    begin
     hr:=D3DWinDevice.HandlePaint(Canvas.Handle);
      if hr<>DD_OK then
       ShowMessage('HandlePaint failed')
      else
       D3DWinDevice:=nil;
    end;
  end;
end;

procedure TForm1.CreateDevice;
var
 hr     : HResult;
 Bpp    : Integer;
begin
 hr:=DirectDrawCreateClipper(0, FD3DClipper, nil);
 if(hr<>D3DRM_OK) then
  ShowMessage('DirectDrawCreateClipper failed');
 FD3DClipper.SetHWnd(0, Handle);
 hr:=FDirect3D.CreateDeviceFromClipper(FD3DClipper, nil, ClientWidth, ClientHeight, FD3Device);
 if(hr<>D3DRM_OK) then
  ShowMessage('CreateDeviceFromClipper failed');

 FD3Device.SetQuality(D3DRMRENDER_GOURAUD);
 bpp:=GetDeviceCaps(Canvas.Handle, BITSPIXEL);

 case (bpp) of
  8: FD3Device.SetDither(false);
  16:
   begin
    FD3Device.SetShades(32);
    FDirect3D.SetDefaultTextureColors(64);
    FDirect3D.SetDefaultTextureShades(32);
    FD3Device.SetDither(false);
   end;
  24, 32:
   begin
    FD3Device.SetShades(256);
    FDirect3D.SetDefaultTextureColors(64);
    FDirect3D.SetDefaultTextureShades(256);
    FD3Device.SetDither(false);
   end;
 end;

 hr:=FDirect3D.CreateFrame(nil, FD3DScene);
 if(hr<>D3DRM_OK) then
  ShowMessage('CreateFrame failed');
 CreateScene;
end;

procedure TForm1.CreateScene;
var
 D3DAmbientLight        : IDirect3DRMLight;
 D3DAmbientLightFrame   : IDirect3DRMFrame;
begin
 if not (LoadScene()) then
  Exit;

 FD3DTextureWrapType:=D3DRMWRAP_FLAT;
 if not (LoadWrapTexture()) then
  Exit;

 FDirect3D.CreateLightRGB(D3DRMLIGHT_AMBIENT, 1, 1, 1, D3DAmbientLight);
 FDirect3D.CreateFrame(FD3DScene,  D3DAmbientLightFrame);
 D3DAmbientLightFrame.AddLight(D3DAmbientLight);
 D3DAmbientLightFrame:=nil;
 D3DAmbientLight:=nil;

 FDirect3D.CreateFrame(FD3DScene, FD3DCamera);
 FD3DCamera.SetPosition(FD3DScene, 0, 0, -50);

 FDirect3D.CreateViewport(FD3Device, FD3DCamera,0, 0, FD3Device.GetWidth(), FD3Device.GetHeight(), FD3DViewPort);

 Flat2Click(nil);
 FD3DScene.SetSceneBackgroundRGB(0, 0, 1);
end;

procedure TForm1.ScaleScene(Mesh: IDirect3DRMMeshBuilder; Dim: D3DVALUE);
var
 LargeDim, SizeX, SizeY, SizeZ: D3DValue;
 D3DBox         : D3DRMBOX;
 ScaleFactor    : D3DVALUE;
begin
 mesh.GetBox(D3DBox);
 Sizex:=D3DBox.max.x-D3DBox.min.x;
 Sizey:=D3DBox.max.y-D3DBox.min.y;
 Sizez:=D3DBox.max.z-D3DBox.min.z;

 Largedim:= 0;
 if (sizex>largedim) then
  largedim:=sizex;
 if (sizey>largedim) then
  largedim:=sizey;
 if (sizez>largedim) then
  largedim:=sizez;
 if LargeDim<>0 then
  Scalefactor:=dim/largedim
 else
  ScaleFactor:=1;

 mesh.Scale(scalefactor, scalefactor, scalefactor);
end;

procedure TForm1.FormResize(Sender: TObject);
var
 View_Width, View_Height: Integer;
 Old_Dither     : LONgBool;
 Old_Quality    :  D3DRMRENDERQUALITY;
 Old_Shades     : Integer;
begin
 if FD3Device=nil then
  Exit;

 if(ClientWidth<>0) and (ClientHeight<>0) then
  begin
   view_width := FD3DViewPort.GetWidth();
   view_height := FD3DViewPort.GetHeight();

   if( view_width=ClientWidth) and (view_height=ClientHeight) then
    Exit;

   old_dither:=FD3Device.GetDither;
   old_quality:=FD3Device.GetQuality();
   old_shades:=FD3Device.GetShades();

   FD3DViewPort:=nil;
   FD3Device:=nil;
   FDirect3D.CreateDeviceFromClipper(FD3DClipper, nil, ClientWidth, ClientHeight, FD3Device );

   FD3Device.SetDither(old_dither);
   FD3Device.SetQuality(old_quality);
   FD3Device.SetShades(old_shades);

   FDirect3D.CreateViewport(FD3Device, FD3DCamera,0,0,FD3Device.GetWidth(), FD3Device.GetHeight(),FD3DViewPort);
  end;
end;

function TForm1.LoadScene: Boolean;
begin
 if(OpenDialog1.Execute()) then
  begin
   FDirect3D.CreateMeshBuilder(FD3DMeshBuilder);
   FD3DMeshBuilder.Load(PChar(OpenDialog1.FileName), nil, D3DRMLOAD_FROMFILE, nil, nil);
   FD3DMeshBuilder.SetPerspective(true);
   ScaleScene(FD3DMeshBuilder, 10);

   FDirect3D.CreateFrame(FD3DScene, FD3DMeshBuilderFrame);
   FD3DMeshBuilderFrame.SetRotation(FD3DScene, 1, 0, 0, 0.1);
   FD3DMeshBuilderFrame.AddVisual(FD3DMeshBuilder);

   Result:=true;
   Exit;
  end;
 Result:=false;
end;

function TForm1.LoadWrapTexture: Boolean;
var
 D3DTexture: IDirect3DRMTexture;
begin
 if (OpenDialog2.Execute()) then
  begin
   FDirect3D.LoadTexture(PChar(OpenDialog2.FileName), D3DTexture);
   FD3DMeshBuilder.SetTexture(D3DTexture);
   FD3DMeshBuilder.SetPerspective(true);
   D3DTexture:=nil;
   Result:=true;
   Exit;
  end;
 Result := false;
end;

procedure TForm1.Flat1Click(Sender: TObject);
begin
 if (FD3DMeshBuilder<>nil) then
  FD3DMeshBuilder.SetQuality(D3DRMRENDER_FLAT);
 Flat1.Checked:=true;
end;

procedure TForm1.Wireframe1Click(Sender: TObject);
begin
 if FD3DMeshBuilder<>nil then
  FD3DMeshBuilder.SetQuality(D3DRMRENDER_WIREFRAME);
 Wireframe1.Checked:=true;
end;

procedure TForm1.Gouraud1Click(Sender: TObject);
begin
 if (FD3DMeshBuilder<>nil) then
  FD3DMeshBuilder.SetQuality(D3DRMRENDER_GOURAUD);
 Gouraud1.Checked:=true;
end;

procedure TForm1.Exit1Click(Sender: TObject);
begin
 Close();
end;

procedure TForm1.ApplyFlat(MeshBuilder: IDirect3DRMMeshBuilder);
var
 Box            : D3DRMBOX;
 Width, Height  : D3DValue;
 Wrap           : IDirect3DRMWrap;
begin
 MeshBuilder.GetBox(Box);
 Width:=box.max.x-box.min.x;
 Height:=box.max.y-box.min.y;

 FDirect3D.CreateWrap(D3DRMWRAP_FLAT, nil,
   0.0, 0.0, 0.0, 0.0, 0.0, 1.0,
   0.0, 1.0, 0.0, 0.5, 0.5,
   D3DDivide(1,width), D3DDivide(1,height), wrap);

 Wrap.Apply(MeshBuilder);
 Wrap:=nil;
end;

procedure TForm1.ApplyCylinder(MeshBuilder: IDirect3DRMMeshBuilder);
var
 Box    : D3DRMBOX;
 Height : D3DValue;
 Wrap   : IDirect3DRMWrap;
begin
 MeshBuilder.GetBox(Box);
 Height:=box.max.y-box.min.y;

 FDirect3D.CreateWrap(D3DRMWRAP_CYLINDER, nil,
   0.0, 0.0, 0.0, 0.0, 0.0, 1.0,
   0.0, 1.0, 0.0, 0.5, 0.5,
   1.0, D3DDivide(1, height), Wrap);

 Wrap.Apply(MeshBuilder);
 Wrap:=nil;
end;

procedure TForm1.ApplySphere(MeshBuilder: IDirect3DRMMeshBuilder);
var
 Wrap: IDirect3DRMWrap;
begin
 FDirect3D.CreateWrap(D3DRMWRAP_SPHERE, nil,
   0.0, 0.0, 0.0, 0.0, 0.0, 1.0,
   0.0, 1.0, 0.0, 0.0, 0.0, 1.0, 1.0, Wrap);

 Wrap.Apply(MeshBuilder);
 Wrap:=nil;
end;


procedure TForm1.Flat2Click(Sender: TObject);
begin
 FD3DTextureWrapType:=D3DRMWRAP_FLAT;
 ApplyFlat(FD3DMeshBuilder);
 Flat2.Checked:=true;
end;

procedure TForm1.Cylinder1Click(Sender: TObject);
begin
 FD3DTextureWrapType:=D3DRMWRAP_CYLINDER;
 ApplyCylinder(FD3DMeshBuilder);
 Cylinder1.Checked:=true;
end;

procedure TForm1.Sphere1Click(Sender: TObject);
begin
 FD3DTextureWrapType:=D3DRMWRAP_SPHERE;
 ApplySphere(FD3DMeshBuilder);
 Sphere1.Checked:=true;
end;

procedure TForm1.ChangleBackgroundColor1Click(Sender: TObject);
var
 r, g, b: D3DValue;
begin
 if (ColorDialog1.Execute()) then
  begin
   r:=GetRValue(ColorDialog1.Color) div 255;
   g:=GetGValue(ColorDialog1.Color) div 255;
   b:=GetBValue(ColorDialog1.Color) div 255;
   FD3DScene.SetSceneBackgroundRGB(r, g, b);
  end;
end;

procedure TForm1.X1Click(Sender: TObject);
begin
 FD3DMeshBuilderFrame.SetRotation(FD3DScene, 1, 0, 0, 0.1);
 X1.Checked:=true;
end;

procedure TForm1.Y1Click(Sender: TObject);
begin
 FD3DMeshBuilderFrame.SetRotation(FD3DScene, 0, 1, 0, 0.1);
 Y1.Checked:=true;
end;

procedure TForm1.Z1Click(Sender: TObject);
begin
 FD3DMeshBuilderFrame.SetRotation(FD3DScene, 0, 0, 1, 0.1);
 Z1.Checked:=true;
end;

procedure TForm1.Stop1Click(Sender: TObject);
begin
 FD3DMeshBuilderFrame.SetRotation(FD3DScene, 0, 0, 0, 0);
 Stop1.Checked:=true;
end;

procedure TForm1.FormActivate(Sender: TObject);
var
  D3DWinDevice  : IDirect3DRMWinDevice;
  hr            : HResult;
begin
 if FD3Device<>nil then
  begin
   hr:=FD3Device.QueryInterface(IID_IDirect3DRMWinDevice, D3DWinDevice);
   if (hr=DD_OK) then
    begin
     if(Application.Active) then
      begin
       hr:=D3DWinDevice.HandleActivate(MAKELONG(WA_ACTIVE, 0));
       if hr<>DD_OK then
        ShowMessage('D3DWinDevice failed')
      end
     else
      begin
       hr:=D3DWinDevice.HandleActivate(MAKELONG(WA_INACTIVE, 0));
       if hr<>DD_OK then
        ShowMessage('HandleActivate failed');
      end;
    end;
   D3DWinDevice:=nil;
  end;
end;

end.
